@@ -1,5 +1,6 @@
-# (C) Copyright 2008 Nuxeo SAS <http://nuxeo.com>
+# (C) Copyright 2009 Nuxeo SAS <http://nuxeo.com>
 # Author: bdelbosc@nuxeo.com
+# Contributors: Kelvin Ward
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as published
@@ -17,10 +18,12 @@
 #
 """Render chart using gnuplot >= 4.2
 
-$Id: ReportRenderHtmlGnuPlot.py 53544 2009-03-09 16:28:58Z tlazar $
+$Id: ReportRenderHtmlGnuPlot.py 54037 2009-11-25 20:16:17Z bdelbosc $
 """
 
 import os
+import sys
+import re
 from commands import getstatusoutput
 from ReportRenderRst import rst_title
 from ReportRenderHtmlBase import RenderHtmlBase
@@ -30,11 +33,32 @@ from datetime import datetime
 def gnuplot(script_path):
     """Execute a gnuplot script."""
     path = os.path.dirname(os.path.abspath(script_path))
-    cmd = 'cd ' + path + '; gnuplot ' + os.path.abspath(script_path)
-    ret, output = getstatusoutput(cmd)
-    if ret != 0:
-        raise RuntimeError("Failed to run gnuplot cmd: " + cmd +
-                           "\n" + str(output))
+    if 'win' in sys.platform.lower():
+        # commands module doesn't work on win and gnuplot is named
+        # wgnuplot
+        ret = os.system('cd "' + path + '" && wgnuplot "' +
+                        os.path.abspath(script_path) + '"')
+        if ret != 0:
+            raise RuntimeError("Failed to run wgnuplot cmd on " +
+                               os.path.abspath(script_path))
+
+    else:
+        cmd = 'cd ' + path + '; gnuplot ' + os.path.abspath(script_path)
+        ret, output = getstatusoutput(cmd)
+        if ret != 0:
+            raise RuntimeError("Failed to run gnuplot cmd: " + cmd +
+                               "\n" + str(output))
+
+def gnuplot_scriptpath(base, filename):
+    """Return a file path string from the join of base and file name for use
+    inside a gnuplot script.
+
+    Backslashes (the win os separator) are replaced with forward
+    slashes. This is done because gnuplot scripts interpret backslashes
+    specially even in path elements.
+    """
+    return os.path.join(base, filename).replace("\\", "/")
+
 
 class RenderHtmlGnuPlot(RenderHtmlBase):
     """Render stats in html using gnuplot
@@ -43,6 +67,7 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
     """
     chart_size = (480, 480)
     #big_chart_size = (640, 480)
+    ticpattern = re.compile('(\:\d+)\ ')
 
     def getChartSizeTmp(self, cvus):
         """Override for gnuplot format"""
@@ -57,11 +82,38 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
             maxCycle = maxCycle[1:]
         return "[0:" + maxCycle + "]"
 
+    def useXTicLabels(self):
+        """Guess if we need to use labels for x axis or number."""
+        cycles = self.config['cycles'][1:-1].split(',')
+        if len(cycles) <= 1:
+            # single cycle
+            return True
+        if len(cycles) != len(set(cycles)):
+            # duplicates cycles
+            return True
+        cycles = [int(i) for i in cycles]
+        for i, v in enumerate(cycles[1:]):
+            # unordered cycles
+            if cycles[i] > v:
+                return True
+        return False
+
+    def fixXLabels(self, lines):
+        """Fix gnuplot script if CUs are not ordered."""
+        if not self.useXTicLabels():
+            return lines
+        # remove xrange line
+        out = lines.replace('set xrange', '#set xrange')
+        # rewrite plot using xticlabels
+        out = out.replace(' 1:', ' :')
+        out = self.ticpattern.sub(r'\1:xticlabels(1) ', out)
+        return out
+
     def createTestChart(self):
         """Create the test chart."""
-        image_path = str(os.path.join(self.report_dir, 'tests.png'))
+        image_path = gnuplot_scriptpath(self.report_dir, 'tests.png')
         gplot_path = str(os.path.join(self.report_dir, 'tests.gplot'))
-        data_path = str(os.path.join(self.report_dir, 'tests.data'))
+        data_path  = gnuplot_scriptpath(self.report_dir, 'tests.data')
         stats = self.stats
         # data
         lines = ["CUs STPS ERROR"]
@@ -91,6 +143,7 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
         lines.append('set ylabel "Test/s"')
         lines.append('set grid back')
         lines.append('set xrange ' + self.getXRange())
+
         if not has_error:
             lines.append('plot "%s" u 1:2 w linespoints lw 2 lt 2 t "STPS"' % data_path)
         else:
@@ -116,7 +169,8 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
             lines.append('plot "%s" u 1:3 w linespoints lt 1 lw 2 t "%% Errors"' % data_path)
             lines.append('unset multiplot')
         f = open(gplot_path, 'w')
-        f.write('\n'.join(lines) + '\n')
+        lines = self.fixXLabels('\n'.join(lines) + '\n')
+        f.write(lines)
         f.close()
         gnuplot(gplot_path)
         return
@@ -135,10 +189,10 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
 
     def createPageChart(self):
         """Create the page chart."""
-        image_path = str(os.path.join(self.report_dir, 'pages_spps.png'))
-        image2_path = str(os.path.join(self.report_dir, 'pages.png'))
+        image_path = gnuplot_scriptpath(self.report_dir, 'pages_spps.png')
+        image2_path = gnuplot_scriptpath(self.report_dir, 'pages.png')
         gplot_path = str(os.path.join(self.report_dir, 'pages.gplot'))
-        data_path = str(os.path.join(self.report_dir, 'pages.data'))
+        data_path = gnuplot_scriptpath(self.report_dir, 'pages.data')
         stats = self.stats
         # data
         lines = ["CUs SPPS ERROR MIN AVG MAX P10 P50 P90 P95"]
@@ -207,16 +261,17 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
         lines.append('set style fill solid .25')
         lines.append('plot "%s" u 1:8:8:10:9 t "med/p90/p95" w candlesticks lt 1 lw 1 whiskerbars 0.5, "" u 1:7:4:8:8 w candlesticks lt 2 lw 1 t "min/p10/med" whiskerbars 0.5, "" u 1:5 t "avg" w lines lt 3 lw 2' % data_path)
         f = open(gplot_path, 'w')
-        f.write('\n'.join(lines) + '\n')
+        lines = self.fixXLabels('\n'.join(lines) + '\n')
+        f.write(lines)
         f.close()
         gnuplot(gplot_path)
 
     def createAllResponseChart(self):
         """Create global responses chart."""
-        image_path = str(os.path.join(self.report_dir, 'requests_rps.png'))
-        image2_path = str(os.path.join(self.report_dir, 'requests.png'))
+        image_path = gnuplot_scriptpath(self.report_dir, 'requests_rps.png')
+        image2_path = gnuplot_scriptpath(self.report_dir, 'requests.png')
         gplot_path = str(os.path.join(self.report_dir, 'requests.gplot'))
-        data_path = str(os.path.join(self.report_dir, 'requests.data'))
+        data_path = gnuplot_scriptpath(self.report_dir, 'requests.data')
         stats = self.stats
         # data
         lines = ["CUs RPS ERROR MIN AVG MAX P10 P50 P90 P95"]
@@ -286,7 +341,8 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
         lines.append('set style fill solid .25')
         lines.append('plot "%s" u 1:8:8:10:9 t "med/p90/p95" w candlesticks lt 1 lw 1 whiskerbars 0.5, "" u 1:7:4:8:8 w candlesticks lt 2 lw 1 t "min/p10/med" whiskerbars 0.5, "" u 1:5 t "avg" w lines lt 3 lw 2' % data_path)
         f = open(gplot_path, 'w')
-        f.write('\n'.join(lines) + '\n')
+        lines = self.fixXLabels('\n'.join(lines) + '\n')
+        f.write(lines)
         f.close()
         gnuplot(gplot_path)
 
@@ -295,10 +351,12 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
 
     def createResponseChart(self, step):
         """Create responses chart."""
-        image_path = str(os.path.join(self.report_dir,
-                                      'request_%s.png' % step))
-        gplot_path = str(os.path.join(self.report_dir, 'request_%s.gplot' % step))
-        data_path = str(os.path.join(self.report_dir, 'request_%s.data' % step))
+        image_path = gnuplot_scriptpath(self.report_dir,
+                                        'request_%s.png' % step)
+        gplot_path = str(os.path.join(self.report_dir,
+                                      'request_%s.gplot' % step))
+        data_path = gnuplot_scriptpath(self.report_dir,
+                                       'request_%s.data' % step)
         stats = self.stats
         # data
         lines = ["CUs STEP ERROR MIN AVG MAX P10 P50 P90 P95"]
@@ -365,7 +423,8 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
             lines.append('unset multiplot')
             lines.append('set size 1.0, 1.0')
         f = open(gplot_path, 'w')
-        f.write('\n'.join(lines) + '\n')
+        lines = self.fixXLabels('\n'.join(lines) + '\n')
+        f.write(lines)
         f.close()
         gnuplot(gplot_path)
         return
@@ -451,10 +510,10 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
                               (1024 * (float(stats[i].time) -
                                        float(stats[i-1].time))))
 
-        image_path = str(os.path.join(self.report_dir,
-                                      '%s_monitor.png' % host))
-        data_path = str(os.path.join(self.report_dir,
-                                     '%s_monitor.data' % host))
+        image_path = gnuplot_scriptpath(self.report_dir,
+                                        '%s_monitor.png' % host)
+        data_path = gnuplot_scriptpath(self.report_dir,
+                                       '%s_monitor.data' % host)
         gplot_path = str(os.path.join(self.report_dir,
                                       '%s_monitor.gplot' % host))
 
